#include "bits/stdc++.h"

using namespace std;

constexpr int W_MAX = 100;
constexpr int H_MAX = 8;
constexpr int S1_MAX = 5;
constexpr int S2_MAX = 4;
constexpr int S3_MAX = 3;
constexpr int MOD = 1'000'000'000 + 7;
constexpr int MAX_STATES = 2400;

using mask = uint32_t;

// dp[0] is start state
int dp[2][H_MAX][S1_MAX + 1][S2_MAX + 1][S3_MAX + 1][MAX_STATES];
int field[H_MAX][W_MAX];
bool fieldS1[H_MAX][W_MAX], fieldS2R[H_MAX][W_MAX], fieldS2D[H_MAX][W_MAX], fieldS3R[H_MAX][W_MAX], fieldS3D[H_MAX][W_MAX];

mask zeroM[H_MAX][MAX_STATES], dec1[H_MAX][MAX_STATES];
mask pr1[H_MAX][MAX_STATES], pr2[H_MAX][MAX_STATES], pr3[H_MAX][MAX_STATES];
mask pd2[H_MAX][MAX_STATES], pd3[H_MAX][MAX_STATES];
int w, h;

uint8_t getMaskElement(mask m, int pos) {
    return (m >> (pos * 2)) & 0b11;
}

mask dec1MaskElement(mask m, int pos) {
    if (getMaskElement(m, pos) == 0) return m;
    return m - (1 << (2 * pos));
}

mask zeroMaskElement(mask m, int pos) {
    return m & (~(0b11 << (2 * pos)));
}

bool maskS1(mask m, int row) {
    return (row == 0 || getMaskElement(m, row - 1) == 0) &&
        getMaskElement(m, row) == 0 &&
            getMaskElement(m, row + 1) <= 1;
}

bool maskS2Right(mask m, int row) {
    return (row == 0 || getMaskElement(m, row - 1) == 0) &&
        getMaskElement(m, row) == 0 &&
            getMaskElement(m, row + 1) <= 1;
}

bool maskS3Right(mask m, int row) {
    return (row == 0 || getMaskElement(m, row - 1) == 0) &&
        getMaskElement(m, row) == 0 &&
            getMaskElement(m, row + 1) <= 1;
}

bool maskS2Down(mask m, int row) {
    return row + 1 < H_MAX && (row == 0 || getMaskElement(m, row - 1) == 0) &&
        ((m >> (2 * row)) & 0xF) == 0 &&
            getMaskElement(m, row + 2) <= 1;
}

bool maskS3Down(mask m, int row) {
    return row + 2 < H_MAX && (row == 0 || getMaskElement(m, row - 1) == 0) &&
        ((m >> (2 * row)) & 0x3F) == 0 &&
            getMaskElement(m, row + 3) <= 1;
}

// Masks are updated as if profile break is moved down.

mask putShipRight(mask m, int i, int len) {
    return m | (len << (2 * i));
}

mask putShipS2Down(mask m, int i) {
    return m | (0b0101 << (2 * i));
}

mask putShipS3Down(mask m, int i) {
    return m | (0b010101 << (2 * i));
}

void addMod(int &dst, int src) {
    dst += src;
    if (dst >= MOD) {
        dst -= MOD;
    }
}

bitset<1 << 16> used[H_MAX];
int maskToID[H_MAX][1 << 16];

void calc_states(int j, mask m) {
    if (used[j][m]) {
        return;
    }
    int j1 = (j + 1) % h;
    int j2 = (j + 2) % h;
    int j3 = (j + 3) % h;
    used[j][m] = true;
    // Maybe we put free cell here?

    if (getMaskElement(m, j) <= 1) {
        mask m1 = zeroMaskElement(m, j);
        calc_states(j1, m1);
    }

    if (getMaskElement(m, j) > 1) {
        // > 1 to really continue
        mask m1 = dec1MaskElement(m, j);
        calc_states(j1, m1);
    }
    // now new ships only
    // ship 1
    if (maskS1(m, j)) {
        mask m1 = putShipRight(m, j, 1);
        calc_states(j1, m1);
    }
    // ship 2
    if (maskS2Right(m, j)) {
        mask m1 = putShipRight(m, j, 2);
        calc_states(j1, m1);
    }
    if (maskS2Down(m, j)) {
        mask m1 = putShipS2Down(m, j);
        calc_states(j2, m1);
    }
    // ship 3
    if (maskS3Right(m, j)) {
        mask m1 = putShipRight(m, j, 3);
        calc_states(j1, m1);
    }
    if (maskS3Down(m, j)) {
        mask m1 = putShipS3Down(m, j);
        calc_states(j3, m1);
    }
}

void solve() {
    cin >> w >> h;
    calc_states(0, 0);
    vector<vector<mask>> idToMask(h);
    for (int i = 0; i < h; i++) {
        for (mask m = 0; m < (1 << 16); m++) {
            if (used[i][m]) {
                idToMask[i].push_back(m);
                maskToID[i][m] = static_cast<int>(idToMask[i].size()) - 1;
            } else {
                maskToID[i][m] = -1;
            }
        }
    }
    for (int i = 0; i < h; i++) {
        string s;
        cin >> s;
        for (int j = 0; j < w; j++) {
            if (s[j] == '.') {
                field[i][j] = 0;
            } else if (s[j] == 'o') {
                field[i][j] = 1; // definitely free
            } else {
                field[i][j] = 2; // definitely occupied
            }
        }
    }

    for (int i = 0; i < h; i++) {
        for (int j = 0; j < w; j++) {
            fieldS1[i][j] = (field[i][j] != 1);
            fieldS2D[i][j] = i + 1 < h &&
                field[i][j] != 1 && field[i + 1][j] != 1;
            fieldS3D[i][j] = i + 2 < h &&
                field[i][j] != 1 && field[i + 1][j] != 1 && field[i + 2][j] != 1;
            fieldS2R[i][j] = j + 1 < w &&
                field[i][j] != 1 && field[i][j + 1] != 1;
            fieldS3R[i][j] = j + 2 < w &&
                field[i][j] != 1 && field[i][j + 1] != 1 && field[i][j + 2] != 1;
        }
    }

    for (int j = 0; j < h; j++) {
        int j1 = (j + 1) % h;
        int j2 = (j + 2) % h;
        int j3 = (j + 3) % h;
        for (int id = 0; id < idToMask[j].size(); id++) {
            mask m = idToMask[j][id];
            zeroM[j][id] = maskToID[j1][zeroMaskElement(m, j)];
            dec1[j][id] = maskToID[j1][dec1MaskElement(m, j)];
            pr1[j][id] = maskToID[j1][putShipRight(m, j, 1)];
            pr2[j][id] = maskToID[j1][putShipRight(m, j, 2)];
            pr3[j][id] = maskToID[j1][putShipRight(m, j, 3)];

            pd2[j][id] = maskToID[j2][putShipS2Down(m, j)];
            pd3[j][id] = maskToID[j3][putShipS3Down(m, j)];

        }
    }
    int s1, s2, s3;
    cin >> s1 >> s2 >> s3;
    dp[0][0][0][0][0][0] = 1; // initial value
    for (int col = 0; col < w; col++) {
        int i = col & 1;
        for (int firstFree = 0; firstFree < h; firstFree++) { // last step manual
            int nxtI1 = i, nxtI2 = i, nxtI3 = i;
            if (firstFree + 1 == h) {
                nxtI1 ^= 1;
            }
            if (firstFree + 2 == h) {
                nxtI2 ^= 1;
            }
            if (firstFree + 3 == h) {
                nxtI3 ^= 1;
            }
            int nxtJ1 = (firstFree + 1) % h;
            int nxtJ2 = (firstFree + 2) % h;
            int nxtJ3 = (firstFree + 3) % h;

            for (int id = 0; id < idToMask[firstFree].size(); id++) {
                mask m = idToMask[firstFree][id];
                // Maybe we put free cell here?
                if (field[firstFree][col] != 2 && getMaskElement(m, firstFree) <= 1) {
                    for (int u1 = 0; u1 <= s1; u1++) {
                        for (int u2 = 0; u2 <= s2; u2++) {
                            for (int u3 = 0; u3 <= s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI1][nxtJ1][u1][u2][u3][zeroM[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }

                }
                // Maybe we started a ship before and we need to continue?
                if (field[firstFree][col] != 1 && getMaskElement(m, firstFree) > 1) {
                    // > 1 to really continue
                    for (int u1 = 0; u1 <= s1; u1++) {
                        for (int u2 = 0; u2 <= s2; u2++) {
                            for (int u3 = 0; u3 <= s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI1][nxtJ1][u1][u2][u3][dec1[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }

                }
                // now new ships only
                // ship 1
                if (fieldS1[firstFree][col] && maskS1(m, firstFree)) {
                    for (int u1 = 0; u1 < s1; u1++) { // NOTE <
                        for (int u2 = 0; u2 <= s2; u2++) {
                            for (int u3 = 0; u3 <= s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI1][nxtJ1][u1 + 1][u2][u3][pr1[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }

                }
                // ship 2
                if (fieldS2R[firstFree][col] && maskS2Right(m, firstFree)) {
                    for (int u1 = 0; u1 <= s1; u1++) {
                        for (int u2 = 0; u2 < s2; u2++) { // NOTE <
                            for (int u3 = 0; u3 <= s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI1][nxtJ1][u1][u2 + 1][u3][pr2[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }
                }
                if (fieldS2D[firstFree][col] && maskS2Down(m, firstFree)) {
                    for (int u1 = 0; u1 <= s1; u1++) {
                        for (int u2 = 0; u2 < s2; u2++) { // NOTE <
                            for (int u3 = 0; u3 <= s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI2][nxtJ2][u1][u2 + 1][u3][pd2[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }

                }
                // ship 3
                if (fieldS3R[firstFree][col] && maskS3Right(m, firstFree)) {
                    for (int u1 = 0; u1 <= s1; u1++) {
                        for (int u2 = 0; u2 <= s2; u2++) {
                            for (int u3 = 0; u3 < s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI1][nxtJ1][u1][u2][u3 + 1][pr3[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }

                }
                if (fieldS3D[firstFree][col] && maskS3Down(m, firstFree)) {
                    for (int u1 = 0; u1 <= s1; u1++) {
                        for (int u2 = 0; u2 <= s2; u2++) {
                            for (int u3 = 0; u3 < s3; u3++) {
                                int val = dp[i][firstFree][u1][u2][u3][id];
                                int &dst = dp[nxtI3][nxtJ3][u1][u2][u3 + 1][pd3[firstFree][id]];
                                addMod(dst, val);
                            }
                        }
                    }
                }
            }
        }
        for (int j = 0; j < h; j++) {
            for (int u1 = 0; u1 <= s1; u1++) {
                for (int u2 = 0; u2 <= s2; u2++) {
                    for (int u3 = 0; u3 <= s3; u3++) {
                        memset(dp[i][j][u1][u2][u3], 0, MAX_STATES * 4);
                    }
                }
            }
        }
    }
    int sum = 0;
    for (int id = 0; id < idToMask[0].size(); id++) {
        addMod(sum, dp[w & 1][0][s1][s2][s3][id]);
    }
    cout << sum << "\n";
}

int main() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);

    solve();

    return 0;
}
