#include <bits/stdc++.h>
using namespace std;

#define MULTITEST
/// #define LOCAL

struct MatchingDinic {
    struct Edge {
        int to;
        int c;
        int f;

        Edge(int to, int c): to(to), c(c), f(0) {}
    };

    vector<Edge> ed;
    vector<int>* g;
    int n;
    int s;
    int t;
    int l;
    int r;
    vector<int> d;
    vector<int> ptr;

    void add_edge(int u, int v, int c) {
        Edge e(v, c);
        Edge erev(u, 0);
        g[u].push_back(ed.size());
        g[v].push_back(ed.size() + 1);
        ed.push_back(e);
        ed.push_back(erev);
    }

    MatchingDinic(int l = 0, int r = 0): n(l + r + 2), s(l + r)
                                       , t(l + r + 1), l(l), r(r) {
        g = new vector<int>[n];
        for (int i = 0; i < l; ++i) {
            add_edge(s, i, 1);
        }
        for (int i = l; i < l + r; ++i) {
            add_edge(i, t, 1);
        }
        d.resize(n);
        ptr.resize(n);
    }

    ~MatchingDinic() {
        delete[] g;
    }

    void add(int u, int v) {
        add_edge(u, v + l, 1);
    }

    bool bfs() {
        fill(d.begin(), d.end(), n);
        fill(ptr.begin(), ptr.end(), 0);
        deque<int> q;
        q.push_back(s);
        d[s] = 0;
        while (!q.empty()) {
            int v = q.front();
            q.pop_front();
            for (int e : g[v]) {
                if (ed[e].c - ed[e].f == 0) continue;
                int u = ed[e].to;
                if (d[u] > d[v] + 1) {
                    d[u] = d[v] + 1;
                    q.push_back(u);
                }
            }
        }
        return d[t] < n;
    }

    int dfs(int v, int min_f) {
        if (v == t) {
            return min_f;
        }
        int sum = 0;
        for (; ptr[v] < (int)g[v].size(); ++ptr[v]) {
            int e = g[v][ptr[v]];
            int u = ed[e].to;
            if (ed[e].c == ed[e].f || d[u] != d[v] + 1) continue;
            int res = dfs(u, min(min_f, ed[e].c - ed[e].f));
            if (res == 0) continue;
            ed[e].f += res;
            ed[e ^ 1].f -= res;
            min_f -= res;
            sum += res;
            if (min_f == 0 || ed[e].c - ed[e].f > 0) break;
        }
        return sum;
    }

    int find_match() {
        while (bfs()) {
            while (dfs(s, n)) {}
        }
        int ans = 0;
        for (int e : g[s]) {
            ans += ed[e].f;
        }
        return ans;
    }

    vector<int> get_free() {
        bfs();
        vector<int> ans;
        for (int i = 0; i < l; ++i) {
            if (d[i] < n) {
                ans.push_back(i);
            }
        }
        return ans;
    }
};

const int MAXN = 1e5;
vector<int> dp[MAXN];
vector<int> g[MAXN];
vector<array<int, 2>> e[MAXN];

vector<int> ord;
int used[MAXN];

void build_ord(int v) {
    used[v] = true;
    int id = -1;
    for (int i = 0; i < (int)g[v].size(); ++i) {
        if (used[g[v][i]]) {
            id = i;
            continue;
        }
        build_ord(g[v][i]);
    }
    ord.push_back(v);
    if (id != -1) {
        swap(g[v][id], g[v][(int)g[v].size() - 1]);
        g[v].pop_back();
    }
}

void solve() {
    int n;
    cin >> n;
    for (int i = 0; i < n; ++i) {
        g[i].clear();
    }
    for (int i = 1; i < n; ++i) {
        int u, v;
        cin >> u >> v;
        u--; v--;
        g[u].push_back(v);
        g[v].push_back(u);
    }
    int m;
    cin >> m;
    for (int i = 0; i < m; ++i) {
        e[i].clear();
    }
    for (int i = 1; i < m; ++i) {
        int u, v;
        cin >> u >> v;
        u--; v--;
        e[u].push_back({v, 2 * i - 2});
        e[v].push_back({u, 2 * i - 1});
    }
    for (int i = 0; i < n; ++i) {
        dp[i].clear();
        dp[i].resize(2 * m, 0);
    }

    ord.clear();
    fill(used, used + n, 0);
    build_ord(0);

    for (int v : ord) {
        for (int u = 0; u < m; ++u) {
            int lft = e[u].size(), rgh = g[v].size();
            if (rgh < lft - 1) {
                continue;
            }
            MatchingDinic match(lft, rgh);
            for (int i = 0; i < rgh; ++i) {
                for (int j = 0; j < lft; ++j) {
                    if (dp[g[v][i]][e[u][j][1]]) {
                        match.add(j, i);
                    }
                }
            }
            int val = match.find_match();
            if (val == lft) {
                cout << "YES\n";
                return;
            } else if (val < lft - 1) {
                continue;
            }
            auto arr = match.get_free();
            for (int j : arr) {
                dp[v][e[u][j][1] ^ 1] = 1;
            }
        }
    }
    cout << "NO\n";
}

main() {
    ios_base::sync_with_stdio(false);
    cin.tie(0); cout.tie(0);
    int tst = 1;
#ifdef MULTITEST
    cin >> tst;
#endif // MULTITEST
    while (tst--) {
        solve();
#ifdef LOCAL
        cout.flush();
#endif // LOCAL
    }
    return 0;
}
