#include "testlib.h"
#include <string>
#include <vector>
using namespace std;

// This function receives stream as an argument, reads an answer from it,
// checks its correctness (i. e. that it is indeed a correct path from s to t in the graph),
// calculates its value and returns it. If the path is incorrect, it stops the execution
// with _wa outcome if stream = ouf (contestant) or with _fail outcome if stream = ans (jury).

vector<int> pi(string &s) {
    int n = (int)s.size();
    vector<int> pref(n);
    int j;
    for (int i = 1; i < n; i++) {
        j = pref[i - 1];
        while (j > 0 && s[i] != s[j]) j = pref[j - 1];
        if (s[i] == s[j]) j++;
        pref[i] = j;
    }
    return pref;
}

bool contains(string &s1, string &s2) {
    int x = (int) s1.size(), y = (int) s2.size();
    if (x > y) return false;
    string s = s1 + '#' + s2;
    vector<int> p = pi(s);
    for (int i = x + 1; i <= x + y; i++) {
        if (p[i] == x) return true;
    }
    return false;
}

bool checkAns(vector<string> &a, string pans, string jans) {
    if (pans.size() > jans.size()) return false;
    for (auto i : a) {
        if (!contains(i, pans)) return false;
    }
    return true;
}

int main(int argc, char* argv[]) {
    registerTestlibCmd(argc, argv);
    int testCount = inf.readInt(1, 100, "testCount");
    for (int t = 0; t < testCount; t++) {
        setTestCase(t + 1);
        int n;
        n = inf.readInt();
        vector<string> a(n);
        for (int i = 0 ; i < n ; i++) {
            a[i] = inf.readWord("[a-z]{1,50000}", "s~");
        }
        string jans = ans.readWord("[a-z]{1,1000000}", "s~");
        string pans = ouf.readWord("[a-z]{1,1000000}", "s~");
        if (!checkAns(a, pans, jans))
            quitf(_wa, "jury has the better answer: jans = %s, pans = %s\n", jans.c_str(), pans.c_str());
    }
    quitf(_ok, "ok answers");
}